// ******************************************************************************
// @file    boot_stage_3_uart.h
// @author  Milandr Application Team
// @version V0.1.0
// @date    12/02/2024
// @brief   header.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

#ifndef boot_stage_3_uart_H
#define boot_stage_3_uart_H

// Includes
#include "system_MDR32VF0xI_config.h"

#define UART_CLK				(8e6) // HSI

#define BAUD_RATE				9600

#define BR_DEV					30 // Baudrate Deviation [%]
#define WO_ERR_CNT				5

#define CMD_SYNC				'\0'
#define CMD_CR					'\r'
#define CMD_LOAD				'L'
#define CMD_VFY					'Y'
#define CMD_RUN					'R'
#define CMD_BAUD				'B'

#define REPLY_ERR				'E'
#define ERR_CHN					'i'
#define ERR_CMD					'c'
#define ERR_BAUD				'b'
#define ERR_FAULT               'f'

#define REPLY_OK				'K'

#define BAUD_RATE_MIN			(BAUD_RATE * (1.0 - BR_DEV/100.0))
#define BAUD_RATE_MAX			(BAUD_RATE * (1.0 + BR_DEV/100.0))

#define GetDiv(clk, baud)		((float)clk/(16*baud))
#define GetIntDiv(baud)			((uint32_t)(64*GetDiv(UART_CLK, baud) + 0.5))

#define DIV_MIN					GetIntDiv(BAUD_RATE_MAX)
#define DIV_MAX					GetIntDiv(BAUD_RATE_MIN)
#define DIV_RANGE				(DIV_MAX - DIV_MIN)
#define DIV_DLT					(DIV_RANGE/BR_DEV/4 ? DIV_RANGE/BR_DEV/4 : 1)

void uart_set_baud (uint32_t divider);
void uart_send_byte (uint32_t ch);
void uart_wait_busy (void);
uint32_t uart_receive_byte (void);
uint32_t uart_sync_err (void);
uint32_t uart_receive_param (void);

void run_app_uart (void);
void uart_io_init (MDR_PORT_TypeDef *PORT);

#endif
