// ******************************************************************************
// @file    boot_common.c
// @author  Milandr Application Team
// @version V0.1.0
// @date    12/02/2024
// @brief   Commom functions.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"
#include "boot_common.h"
#include "boot_stage_2.h"
#include "boot_stage_3_uart.h"

// ------------------------------------------------------------------------------
__TRAP_HANDLER_ALIGNED __INTERRUPT_MACHINE void Trap_IRQHandler(void)
{
    uint32_t boot_mode;
    boot_mode = (MDR_BKP->LDO & BKP_LDO_MODE_Msk) >> BKP_LDO_MODE_Pos;

    switch (boot_mode)
    {
        case mode_flash:
        {
            while (1); // deadlock.
            break;
        }

        case mode_ram:
        {
            while (1); // deadlock.
            break;
        }

        case mode_uart_0:
        case mode_uart_1:
        {
			uart_send_byte (REPLY_ERR);
            uart_send_byte (ERR_FAULT);
            while (1); // deadlock.
            break;       // unreachable code.
        }

        default:
        {
            while (1); // deadlock.
            break;
        }
    }
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void bkp_set_fpor (void)
{
	uint32_t tmp;

    // Unlock BKP write protection, set fpor to BKP fpor bit.
    // Finally lock BKP write protection.

    MDR_BKP->WPR = 0x8555AAA1;
    tmp = MDR_BKP->LDO;
    tmp &= ~(BKP_LDO_SW_OFF_Msk | BKP_LDO_COVDET_Msk);
    tmp |= BKP_LDO_FPOR;
    MDR_BKP->LDO = tmp;
    MDR_BKP->WPR = 0;
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
inline void __attribute__((always_inline)) bkp_write_mode (uint32_t mode)
{
	uint32_t tmp;

    // Set AND mask over mode, because mode field is 2-bit in BKP.

    mode &= (BKP_LDO_MODE_Msk >> BKP_LDO_MODE_Pos);

    // Unlock BKP write protection, write boot mode to BKP mode field.
    // Finally lock BKP write protection.

    MDR_BKP->WPR = 0x8555AAA1;
    tmp = MDR_BKP->LDO;
    tmp &= ~(BKP_LDO_SW_OFF_Msk | BKP_LDO_COVDET_Msk | BKP_LDO_MODE_Msk);
    tmp |= mode << BKP_LDO_MODE_Pos;
    MDR_BKP->LDO = tmp;
    MDR_BKP->WPR = 0;
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void run_app (uint32_t entry_point)
{
    ((void (*)(void))entry_point)();
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
inline void __attribute__((always_inline)) init_mode_port (MDR_PORT_TypeDef *PORT)
{
    PORT->OE &= ~(PORT_OE_OE1 | PORT_OE_OE0);                           // Clear output, setup as input
    PORT->PWR &= ~(PORT_PWR_PWR1 | PORT_PWR_PWR0);                      // Clear PWR
    PORT->FUNC &= ~(PORT_FUNC_MODE1_Msk | PORT_FUNC_MODE0_Msk);         // Clear Func, setup as PORT
    PORT->ANALOG |= (PORT_ANALOG_ANALOG_EN1 | PORT_ANALOG_ANALOG_EN0);  // Setup as digital
    PORT->PULL |= (PORT_PULL_PULL_DOWN1 | PORT_PULL_PULL_DOWN0);        // On pulldowns
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
inline void __attribute__((always_inline)) deinit_mode_port (MDR_PORT_TypeDef *PORT)
{
    PORT->ANALOG &= ~(PORT_ANALOG_ANALOG_EN1 | PORT_ANALOG_ANALOG_EN0); // Set Analog by default
    PORT->PULL &= ~(PORT_PULL_PULL_DOWN1 | PORT_PULL_PULL_DOWN0);       // Off pulldowns by default
}
// ------------------------------------------------------------------------------


// ------------------------------------------------------------------------------
inline void __attribute__((always_inline)) delay_50us_core_500kHz (void)
{
    asm volatile ( "nop\n nop\n nop\n nop\n nop\n");
    asm volatile ( "nop\n nop\n nop\n nop\n nop\n");
    asm volatile ( "nop\n nop\n nop\n nop\n nop\n");
    asm volatile ( "nop\n nop\n nop\n nop\n nop\n");
    asm volatile ( "nop\n nop\n nop\n nop\n nop\n");
}
// ------------------------------------------------------------------------------
