// ******************************************************************************
// @file    boot_stage_3_flash.c
// @author  Milandr Application Team
// @version V0.1.0
// @date    12/02/2024
// @brief   Functions for work with flash.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"
#include "boot_stage_3_flash.h"
#include "boot_common.h"


// ------------------------------------------------------------------------------
void setup_manufacture_trimings (void)
{
    uint32_t checksum;
    uint32_t i;
    uint32_t chip_id;
    uint32_t tmp;

    rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_FLASH);

    chip_id = (MDR_FLASH->CHIP_ID_CTRL & FLASH_CHIP_ID_CTRL_CHIP_ID_Msk); // ID << 2!

    rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_FLASH);

    checksum = 0;

    switch (chip_id)
    {
        case FLASH_CHIP_ID_CTRL_CHIP_ID215:
        {
            for (i = 0; i < 20; i++)
            {
                checksum += (MDR_FLASH_SPECIAL_FIELDS_ID215->UNIQUE_ID [i]);
            }
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_HSI;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_BG;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_LDO;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_TS;

            // Checksum used only last 8 bit
            checksum &= 0xff;

            //  BKP clock setups in main.
            rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_ADC);

            MDR_BKP->WPR = 0x8555AAA1;

            if (checksum == MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_CS)
            {
                tmp =  MDR_BKP->CLK;
                tmp &= ~(BKP_CLK_HSI_TRIM_Msk);
                tmp |= ((MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_HSI << BKP_CLK_HSI_TRIM_Pos) & BKP_CLK_HSI_TRIM_Msk);
                MDR_BKP->CLK = tmp;

                tmp =  MDR_ADC->ADC_TRIM;
                tmp &= ~(ADC_TRIM_BG_TRIM_Msk);
                tmp |= ((MDR_FLASH_SPECIAL_FIELDS_ID215->TRIM_BG << ADC_TRIM_BG_TRIM_Pos) & ADC_TRIM_BG_TRIM_Msk);
                MDR_ADC->ADC_TRIM = tmp;

                // = ManufactureInfo->TRIM_LDO; // it's not present by Manufacture
                // = ManufactureInfo->TRIM_TS;  // it's not present by Manufacture
            }

            else
            {
                tmp =  MDR_BKP->CLK;
                tmp &= ~(BKP_CLK_HSI_TRIM_Msk);
                tmp |= ((HSI_TRIM_ID215 << BKP_CLK_HSI_TRIM_Pos) & BKP_CLK_HSI_TRIM_Msk);
                MDR_BKP->CLK = tmp;

                tmp =  MDR_ADC->ADC_TRIM;
                tmp &= ~(ADC_TRIM_BG_TRIM_Msk);
                tmp |= ((BG_TRIM_ID215 << ADC_TRIM_BG_TRIM_Pos) & ADC_TRIM_BG_TRIM_Msk);
                MDR_ADC->ADC_TRIM = tmp;
            }

            MDR_BKP->WPR = 0;

            rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_ADC);

            break;
        }

        case FLASH_CHIP_ID_CTRL_CHIP_ID217:
        {
            for (i = 0; i < 20; i++)
            {
                checksum += (MDR_FLASH_SPECIAL_FIELDS_ID217->UNIQUE_ID [i]);
            }
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_HSI;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_BG;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_LDO;
            checksum += MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_TS;

            // Checksum used only last 8 bit
            checksum &= 0xff;

            //  BKP clock setups in main.
            rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_ADC);

            MDR_BKP->WPR = 0x8555AAA1;

            if (checksum == MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_CS)
            {
                tmp =  MDR_BKP->CLK;
                tmp &= ~(BKP_CLK_HSI_TRIM_Msk);
                tmp |= ((MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_HSI << BKP_CLK_HSI_TRIM_Pos) & BKP_CLK_HSI_TRIM_Msk);
                MDR_BKP->CLK = tmp;

                tmp =  MDR_ADC->ADC_TRIM;
                tmp &= ~(ADC_TRIM_BG_TRIM_Msk);
                tmp |= ((MDR_FLASH_SPECIAL_FIELDS_ID217->TRIM_BG << ADC_TRIM_BG_TRIM_Pos) & ADC_TRIM_BG_TRIM_Msk);
                MDR_ADC->ADC_TRIM = tmp;

            // = ManufactureInfo->TRIM_LDO; // it's not present by Manufacture
            // = ManufactureInfo->TRIM_TS;  // it's not present by Manufacture
            }

            else
            {
                tmp =  MDR_BKP->CLK;
                tmp &= ~(BKP_CLK_HSI_TRIM_Msk);
                tmp |= ((HSI_TRIM_ID217 << BKP_CLK_HSI_TRIM_Pos) & BKP_CLK_HSI_TRIM_Msk);
                MDR_BKP->CLK = tmp;

                tmp =  MDR_ADC->ADC_TRIM;
                tmp &= ~(ADC_TRIM_BG_TRIM_Msk);
                tmp |= ((BG_TRIM_ID217 << ADC_TRIM_BG_TRIM_Pos) & ADC_TRIM_BG_TRIM_Msk);
                MDR_ADC->ADC_TRIM = tmp;
            }

            MDR_BKP->WPR = 0;

            rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_ADC);

            break;
        }

        default:
            break;
    }
}
// ------------------------------------------------------------------------------
